# Copyright (C) 2004 Igor Belyi <belyi@users.sourceforge.net>
# Copyright (C) 2002 John Goerzen <jgoerzen@complete.org>
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

from __future__ import absolute_import, print_function, unicode_literals
del absolute_import, print_function, unicode_literals

# GPGME 1.7 replaced gpgme_op_edit with gpgme_op_interact.  We
# implement gpg.Context.op_edit using gpgme_op_interact, so the
# callbacks will be called with string keywords instead of numeric
# status messages.  Code that is using these constants will continue
# to work.

ABORT = "ABORT"
ALREADY_SIGNED = "ALREADY_SIGNED"
ATTRIBUTE = "ATTRIBUTE"
BACKUP_KEY_CREATED = "BACKUP_KEY_CREATED"
BAD_PASSPHRASE = "BAD_PASSPHRASE"
BADARMOR = "BADARMOR"
BADMDC = "BADMDC"
BADSIG = "BADSIG"
BEGIN_DECRYPTION = "BEGIN_DECRYPTION"
BEGIN_ENCRYPTION = "BEGIN_ENCRYPTION"
BEGIN_SIGNING = "BEGIN_SIGNING"
BEGIN_STREAM = "BEGIN_STREAM"
CARDCTRL = "CARDCTRL"
DECRYPTION_FAILED = "DECRYPTION_FAILED"
DECRYPTION_INFO = "DECRYPTION_INFO"
DECRYPTION_OKAY = "DECRYPTION_OKAY"
DELETE_PROBLEM = "DELETE_PROBLEM"
ENC_TO = "ENC_TO"
END_DECRYPTION = "END_DECRYPTION"
END_ENCRYPTION = "END_ENCRYPTION"
END_STREAM = "END_STREAM"
ENTER = "ENTER"
ERRMDC = "ERRMDC"
ERROR = "ERROR"
ERRSIG = "ERRSIG"
EXPKEYSIG = "EXPKEYSIG"
EXPSIG = "EXPSIG"
FAILURE = "FAILURE"
FILE_DONE = "FILE_DONE"
FILE_ERROR = "FILE_ERROR"
FILE_START = "FILE_START"
GET_BOOL = "GET_BOOL"
GET_HIDDEN = "GET_HIDDEN"
GET_LINE = "GET_LINE"
GOOD_PASSPHRASE = "GOOD_PASSPHRASE"
GOODMDC = "GOODMDC"
GOODSIG = "GOODSIG"
GOT_IT = "GOT_IT"
IMPORT_OK = "IMPORT_OK"
IMPORT_PROBLEM = "IMPORT_PROBLEM"
IMPORT_RES = "IMPORT_RES"
IMPORTED = "IMPORTED"
INQUIRE_MAXLEN = "INQUIRE_MAXLEN"
INV_RECP = "INV_RECP"
INV_SGNR = "INV_SGNR"
KEY_CONSIDERED = "KEY_CONSIDERED"
KEY_CREATED = "KEY_CREATED"
KEY_NOT_CREATED = "KEY_NOT_CREATED"
KEYEXPIRED = "KEYEXPIRED"
KEYREVOKED = "KEYREVOKED"
LEAVE = "LEAVE"
MISSING_PASSPHRASE = "MISSING_PASSPHRASE"
MOUNTPOINT = "MOUNTPOINT"
NEED_PASSPHRASE = "NEED_PASSPHRASE"
NEED_PASSPHRASE_PIN = "NEED_PASSPHRASE_PIN"
NEED_PASSPHRASE_SYM = "NEED_PASSPHRASE_SYM"
NEWSIG = "NEWSIG"
NO_PUBKEY = "NO_PUBKEY"
NO_RECP = "NO_RECP"
NO_SECKEY = "NO_SECKEY"
NO_SGNR = "NO_SGNR"
NODATA = "NODATA"
NOTATION_DATA = "NOTATION_DATA"
NOTATION_FLAGS = "NOTATION_FLAGS"
NOTATION_NAME = "NOTATION_NAME"
PINENTRY_LAUNCHED = "PINENTRY_LAUNCHED"
PKA_TRUST_BAD = "PKA_TRUST_BAD"
PKA_TRUST_GOOD = "PKA_TRUST_GOOD"
PLAINTEXT = "PLAINTEXT"
PLAINTEXT_LENGTH = "PLAINTEXT_LENGTH"
POLICY_URL = "POLICY_URL"
PROGRESS = "PROGRESS"
REVKEYSIG = "REVKEYSIG"
RSA_OR_IDEA = "RSA_OR_IDEA"
SC_OP_FAILURE = "SC_OP_FAILURE"
SC_OP_SUCCESS = "SC_OP_SUCCESS"
SESSION_KEY = "SESSION_KEY"
SHM_GET = "SHM_GET"
SHM_GET_BOOL = "SHM_GET_BOOL"
SHM_GET_HIDDEN = "SHM_GET_HIDDEN"
SHM_INFO = "SHM_INFO"
SIG_CREATED = "SIG_CREATED"
SIG_ID = "SIG_ID"
SIG_SUBPACKET = "SIG_SUBPACKET"
SIGEXPIRED = "SIGEXPIRED"
SUCCESS = "SUCCESS"
TOFU_STATS = "TOFU_STATS"
TOFU_STATS_LONG = "TOFU_STATS_LONG"
TOFU_USER = "TOFU_USER"
TRUNCATED = "TRUNCATED"
TRUST_FULLY = "TRUST_FULLY"
TRUST_MARGINAL = "TRUST_MARGINAL"
TRUST_NEVER = "TRUST_NEVER"
TRUST_ULTIMATE = "TRUST_ULTIMATE"
TRUST_UNDEFINED = "TRUST_UNDEFINED"
UNEXPECTED = "UNEXPECTED"
USERID_HINT = "USERID_HINT"
VALIDSIG = "VALIDSIG"
